@@ -30,6 +30,7 @@ import os
 import shutil
 import datetime
 import warnings
+import logging
 
 from sqlalchemy import orm
 
@@ -42,6 +43,9 @@ from rattail.time import localtime, make_utc
 from rattail.util import progress_loop, load_object
 
 
+log = logging.getLogger(__name__)
+
+
 class BatchHandler(object):
     """
     Base class and partial default implementation for batch handlers.  It is
@@ -184,7 +188,8 @@ class BatchHandler(object):
         """
 
     def purge_batches(self, session, before=None, before_days=90,
-                      delete_all_data=True, progress=None, **kwargs):
+                      dry_run=False, delete_all_data=None,
+                      progress=None, **kwargs):
         """
         Purge all batches which were executed prior to a given date.
 
@@ -195,26 +200,40 @@ class BatchHandler(object):
         :param before_days: Number of days before the current date, to be used
            as the cutoff date if ``before`` is not specified.
 
+        :param dry_run: Flag indicating that this is a "dry run" and all logic
+           involved should be (made) aware of that fact.
+
         :param delete_all_data: Flag indicating whether *all* data should be
            deleted for each batch being purged.  This flag is passed along to
-           :meth:`delete()`; see that for more info.
+           :meth:`delete()`; see that for more info.  NOTE: This flag should
+           probably be deprecated, but so far has not been...but ``dry_run``
+           should be preferred for readability etc.
 
         :returns: Integer indicating the number of batches purged.
         """
+        if delete_all_data and dry_run:
+            raise ValueError("You can enable (n)either of `dry_run` or "
+                             "`delete_all_data` but both cannot be True")
+        delete_all_data = not dry_run
+
         if not before:
             before = localtime(self.config).date() - datetime.timedelta(days=before_days)
             before = datetime.datetime.combine(before, datetime.time(0))
             before = localtime(self.config, before)
 
+        log.info("will purge '%s' batches, executed before %s",
+                 self.batch_key, before.date())
+
         old_batches = session.query(self.batch_model_class)\
                              .filter(self.batch_model_class.executed < before)\
-                             .options(orm.joinedload(self.batch_model_class.data_rows))
+                             .options(orm.joinedload(self.batch_model_class.data_rows))\
+                             .all()
+        log.info("found %s batches to purge", len(old_batches))
         result = Object()
         result.purged = 0
 
         def purge(batch, i):
-            self.delete(batch, delete_all_data=delete_all_data, progress=progress)
-            session.delete(batch)
+            self.do_delete(batch, dry_run=dry_run)
             result.purged += 1
             if i % 5 == 0:
                 session.flush()
@@ -223,6 +242,10 @@ class BatchHandler(object):
                            message="Purging old batches")
 
         session.flush()
+        if old_batches:
+            log.info("%spurged %s '%s' batches",
+                     "(would have) " if dry_run else "",
+                     result.purged, self.batch_key)
         return result.purged
 
     @property
@@ -708,20 +731,45 @@ class BatchHandler(object):
                 batch.executed_by = kwargs['user']
         return True
 
+    def do_delete(self, batch, dry_run=False, progress=None, **kwargs):
+        """
+        Totally delete the given batch.  This includes deleting the batch
+        itself, any rows and "extra" data such as files.
+
+        Note that callers *should* use this method, but custom batch handlers
+        should *not* override this method.  Conversely, custom handlers
+        *should* override the :meth:`~delete()` method, but callers should
+        *not* use that one directly.
+        """
+        session = orm.object_session(batch)
+
+        if 'delete_all_data' in kwargs:
+            warnings.warn("The 'delete_all_data' kwarg is not supported for "
+                          "this method; please use 'dry_run' instead",
+                          DeprecationWarning)
+        kwargs['delete_all_data'] = not dry_run
+
+        self.delete(batch, progress=progress, **kwargs)
+        session.delete(batch)
+
     def delete(self, batch, delete_all_data=True, progress=None, **kwargs):
         """
         Delete all data for the batch, including any related (e.g. row)
         records, as well as files on disk etc.  This method should *not* delete
         the batch itself however.
 
+        Note that callers should *not* use this method, but custom batch
+        handlers *should* override this method.  Conversely, custom handlers
+        should *not* override the :meth:`~do_delete()` method, but callers
+        *should* use that one directly.
+
         :param delete_all_data: Flag indicating whether *all* data should be
            deleted.  You should probably set this to ``False`` if in dry-run
            mode, since deleting *all* data often implies deleting files from
            disk, which is not transactional and therefore can't be rolled back.
         """
         if delete_all_data:
-            if hasattr(batch, 'delete_data'):
-                batch.delete_data(self.config)
+            self.delete_extra_data(batch, progress=progress)
 
         # delete all rows from batch, one by one.  maybe would be nicer if we
         # could delete all in one fell swoop, but sometimes "extension" row
@@ -739,6 +787,22 @@ class BatchHandler(object):
             self.progress_loop(delete, batch.data_rows, progress,
                                message="Deleting rows from batch")
 
+            # even though we just deleted all rows, we must also "remove" all
+            # rows explicitly from the batch; otherwise when the batch itself
+            # is deleted, SQLAlchemy may complain about an unexpected number of
+            # rows being deleted
+            del batch.data_rows[:]
+
+    def delete_extra_data(self, batch, progress=None, **kwargs):
+        """
+        Delete all "extra" data for the batch.  This method should *not* bother
+        trying to delete the batch itself, or rows thereof.  It typically is
+        only concerned with deleting extra files on disk, related to the batch.
+        """
+        path = self.config.batch_filepath(self.batch_key, batch.uuid)
+        if os.path.exists(path):
+            shutil.rmtree(path)
+
     def setup_clone(self, oldbatch, progress=None):
         """
         Perform any setup (caching etc.) necessary for cloning batch.  Note
