@@ -311,6 +311,17 @@ Commands:\n""".format(self))
             elif not args.no_versioning:
                 configure_versioning(self.config)
 
+        # import our primary data model now, just in case it hasn't fully been
+        # imported yet.  this it to be sure association proxies and the like
+        # are fully wired up in the case of extensions
+        # TODO: what about web apps etc.? i guess i was only having the problem
+        # for some importers, e.g. basic CORE API -> Rattail w/ the schema
+        # extensions in place from rattail-corepos
+        try:
+            self.config.get_model()
+        except ImportError:
+            pass
+
         # instantiate the subcommand
         subcmd = self.subcommands[cmd](self, self.config)
 
