@@ -32,7 +32,7 @@ from sqlalchemy import orm
 
 from rattail.db import Session, model
 from rattail.db.util import make_topo_sortkey
-from rattail.datasync import DataSyncWatcher, DataSyncConsumer
+from rattail.datasync import DataSyncWatcher, DataSyncConsumer, NewDataSyncImportConsumer
 from rattail.config import parse_list
 
 
@@ -45,28 +45,45 @@ class RattailWatcher(DataSyncWatcher):
     """
     prunes_changes = True
 
+    def __init__(self, *args, **kwargs):
+        super(RattailWatcher, self).__init__(*args, **kwargs)
+        self.engine = self.config.rattail_engines[self.dbkey]
+        self.topo_sortkey = make_topo_sortkey(self.config.get_model())
+
     def get_changes(self, lastrun):
         """
         Checks the :class:`rattail.db.model:Change` table in a Rattail
         database, to see if there are any pending changes for the datasync
         daemon.
         """
-        session = Session()
+        session = Session(bind=self.engine)
         changes = session.query(model.Change).all()
         session.expunge_all()
         session.close()
-        if changes:
-            return [
-                (c.uuid, 
-                 model.DataSyncChange(
-                     payload_type=c.class_name,
-                     payload_key=c.instance_uuid,
-                     deletion=c.deleted))
-                for c in changes]
+        if not changes:
+            return
+
+        # here we sort our payload types (class names) topologically, to take
+        # foreign key dependencies into account etc.  but we start with a
+        # simple lexical sort on class name, just for kicks maybe..?
+        class_names = sorted(set([c.class_name for c in changes]))
+        class_names.sort(key=self.topo_sortkey)
+
+        # now create "final" list of changes, honoring class sort
+        # TODO: should we force deletions to be last? or first?
+        final = []
+        for class_name in class_names:
+            for change in [c for c in changes if c.class_name == class_name]:
+                final.append((change.uuid,
+                              model.DataSyncChange(
+                                  payload_type=class_name,
+                                  payload_key=change.instance_uuid,
+                                  deletion=change.deleted)))
+        return final
 
     def prune_changes(self, keys):
         deleted = 0
-        session = Session()
+        session = Session(bind=self.engine)
         for key in keys:
             change = session.query(model.Change).get(key)
             if change:
@@ -143,6 +160,14 @@ class RattailConsumer(DataSyncConsumer):
                 else: # add/update
                     host_instance = host_session.query(cls).get(change.payload_key)
                     if host_instance:
+
+                        # if processing a pack item, must process its unit item
+                        # first.  this is because both may be "new" in which
+                        # case the unit item must exist before the pack may
+                        # reference it.
+                        if class_name == 'Product' and host_instance.is_pack_item():
+                            self.merge_instance(session, host_instance.unit)
+
                         self.merge_instance(session, host_instance)
                         session.flush()
                     else:
@@ -203,7 +228,10 @@ class RattailConsumer(DataSyncConsumer):
         # force the ORM to notice a change, since the UUID field is ultimately
         # what it's watching.  So we must explicitly use that field here.
         target_product.regular_price_uuid = None
+        target_product.tpr_price_uuid = None
+        target_product.sale_price_uuid = None
         target_product.current_price_uuid = None
+        target_product.suggested_price_uuid = None
 
         # If the source instance has currently valid price relationships, then
         # we re-establish them.  We must merge the source price instance in
@@ -216,9 +244,18 @@ class RattailConsumer(DataSyncConsumer):
         if source_product.regular_price_uuid:
             target_product.regular_price = session.merge(source_product.regular_price)
             target_product.regular_price_uuid = target_product.regular_price.uuid
+        if source_product.tpr_price_uuid:
+            target_product.tpr_price = session.merge(source_product.tpr_price)
+            target_product.tpr_price_uuid = target_product.tpr_price.uuid
+        if source_product.sale_price_uuid:
+            target_product.sale_price = session.merge(source_product.sale_price)
+            target_product.sale_price_uuid = target_product.sale_price.uuid
         if source_product.current_price_uuid:
             target_product.current_price = session.merge(source_product.current_price)
             target_product.current_price_uuid = target_product.current_price.uuid
+        if source_product.suggested_price_uuid:
+            target_product.suggested_price = session.merge(source_product.suggested_price)
+            target_product.suggested_price_uuid = target_product.suggested_price.uuid
 
         return target_product
 
@@ -267,3 +304,97 @@ class RattailConsumer(DataSyncConsumer):
             model.CustomerGroupAssignment.group == group)
         for assignment in q:
             session.delete(assignment)
+
+
+class FromRattailToRattailExportConsumer(NewDataSyncImportConsumer):
+    """
+    Export data changes from "local" Rattail to another
+    """
+    handler_spec = None
+
+    def __init__(self, *args, **kwargs):
+        super(FromRattailToRattailExportConsumer, self).__init__(*args, **kwargs)
+        self.target_engine = self.config.rattail_engines[self.dbkey]
+        self.model = self.config.get_model()
+
+    def setup(self):
+        super(FromRattailToRattailExportConsumer, self).setup()
+        self.topo_sortkey = make_topo_sortkey(self.model)
+
+    def process_changes(self, session, changes):
+        target_session = Session(bind=self.target_engine)
+
+        if self.runas_username:
+            target_session.set_continuum_user(self.runas_username)
+
+        # update all importers with current sessions
+        for importer in self.importers.values():
+            importer.host_session = session
+            importer.session = target_session
+
+        # topographically sort changes, i.e. per schema table dependencies.
+        # we start with a lexical sort just for kicks maybe...
+        class_names = set([c.payload_type for c in changes])
+        class_names = sorted(class_names)
+        class_names.sort(key=self.topo_sortkey)
+
+        for class_name in class_names:
+            cls = getattr(self.model, class_name)
+            for change in [c for c in changes if c.payload_type == class_name]:
+                self.invoke_importer(session, change)
+
+        target_session.commit()
+        target_session.close()
+
+    def get_host_object(self, session, change):
+        cls = getattr(self.model, change.payload_type)
+        return session.query(cls).get(change.payload_key)
+
+
+class FromRattailToRattailImportConsumer(NewDataSyncImportConsumer):
+    """
+    Import data changes from another Rattail to "local"
+    """
+    handler_spec = None
+
+    def __init__(self, *args, **kwargs):
+        super(FromRattailToRattailImportConsumer, self).__init__(*args, **kwargs)
+        self.host_engine = self.config.rattail_engines[self.watcher.dbkey]
+        self.local_engine = self.config.rattail_engines[self.dbkey]
+        self.model = self.config.get_model()
+
+    def setup(self):
+        super(FromRattailToRattailImportConsumer, self).setup()
+        self.topo_sortkey = make_topo_sortkey(self.model)
+
+    def process_changes(self, session, changes):
+        self.host_session = Session(bind=self.host_engine)
+        local_session = Session(bind=self.local_engine)
+        if self.runas_username:
+            local_session.set_continuum_user(self.runas_username)
+
+        # update all importers with current sessions
+        for importer in self.importers.values():
+            importer.host_session = self.host_session
+            importer.session = local_session
+
+        # topographically sort changes, i.e. per schema table dependencies.
+        # we start with a lexical sort just for kicks maybe...
+        class_names = set([c.payload_type for c in changes])
+        class_names = sorted(class_names)
+        class_names.sort(key=self.topo_sortkey)
+
+        for class_name in class_names:
+            cls = getattr(self.model, class_name)
+            for change in [c for c in changes if c.payload_type == class_name]:
+                self.invoke_importer(session, change)
+
+        local_session.commit()
+        local_session.close()
+        # TODO: should we ever commit this..?
+        # self.host_session.commit()
+        self.host_session.close()
+
+    def get_host_object(self, session, change):
+        cls = getattr(self.model, change.payload_type)
+        return self.host_session.query(cls).get(change.payload_key)
