@@ -596,9 +596,27 @@ class Importer(object):
         deletion.  By default this will be all keys from the local cached data
         set, or an empty set if local data isn't cached.
         """
-        if self.caches_local_data and self.cached_local_data is not None:
-            return set(self.cached_local_data)
-        return set()
+        if not self.caches_local_data:
+            return set()
+
+        if self.cached_local_data is None:
+            return set()
+
+        all_keys = set(self.cached_local_data)
+        keys = set()
+
+        def check(key, i):
+            data = self.cached_local_data[key]['data']
+            obj = self.cached_local_data[key]['object']
+            if self.can_delete_object(obj, data):
+                keys.add(key)
+
+        self.progress_loop(check, all_keys,
+                           message="Determining which objects can be deleted")
+        return keys
+
+    def can_delete_object(self, obj, data):
+        return True
 
     def delete_object(self, obj):
         """
