@@ -1289,14 +1289,20 @@ class SubdepartmentImporter(ToRattail):
 
     def update_object(self, subdepartment, data, local_data=None):
         subdepartment = super(SubdepartmentImporter, self).update_object(subdepartment, data, local_data)
+
         if 'department_number' in self.fields:
-            dept = self.get_department(data['department_number'])
-            if not dept:
-                dept = model.Department()
-                dept.number = data['department_number']
-                self.session.add(dept)
-                self.departments[dept.number] = dept
-            subdepartment.department = dept
+            department_number = data['department_number']
+            if department_number is not None:
+                dept = self.get_department(department_number)
+                if not dept:
+                    dept = model.Department()
+                    dept.number = department_number
+                    self.session.add(dept)
+                    self.departments[dept.number] = dept
+                subdepartment.department = dept
+            elif subdepartment.department:
+                subdepartment.department = None
+
         return subdepartment
 
 
