@@ -33,6 +33,8 @@ import six
 
 from rattail.db import model
 from rattail.util import load_entry_points, load_object
+from rattail.mail import send_email
+from rattail.time import localtime
 
 
 class ReportHandler(object):
@@ -93,6 +95,33 @@ class ReportHandler(object):
             params[key] = value
         return params
 
+    def email_output(self, report, output, mailkey,
+                     fallback_mailkey='common_report', extra_data={}, **kwargs):
+        """
+        Send an email (using the given :param:`mailkey`) with the report output
+        as an attachment.
+        """
+        if 'attachments' not in kwargs:
+            path = output.filepath(self.config)
+            kwargs['attachments'] = [path]
+
+        data = {
+            'config': self.config,
+            'handler': self,
+            'report': report,
+            'output': output,
+            'localtime': localtime,
+        }
+
+        if extra_data:
+            data.update(extra_data)
+
+        if 'params' not in data:
+            data['params'] = output.params
+
+        send_email(self.config, mailkey, data, fallback_key=fallback_mailkey,
+                   **kwargs)
+
 
 def get_report_handler(config):
     """
